//=========================================================
//  MusE
//  Linux Music Editor
//  $Id: drummap.cpp,v 1.3.2.4 2009/05/03 04:14:00 terminator356 Exp $
//
//  (C) Copyright 1999/2000 Werner Schweer (ws@seh.de)
//=========================================================

#include "audio.h"
#include "drummap.h"
#include "xml.h"
#include "song.h"

char drumOutmap[DRUM_MAPSIZE];
char drumInmap[128];

DrumMap drumMap[DRUM_MAPSIZE];

//---------------------------------------------------------
//    GM default drum map
//---------------------------------------------------------

const DrumMap blankdm = { QString(""), 16, 32, 9, 0, 70, 90, 110, 127, 127, 127, false };

const DrumMap idrumMap[DRUM_MAPSIZE] = {
      { QString("Acoustic Bass Drum"), 16, 32, 9, 0, 70, 90, 110, 127, 35, 35, false },
      { QString("Bass Drum 1"),        16, 32, 9, 0, 70, 90, 110, 127, 36, 36, false },
      { QString("Side Stick"),         16, 32, 9, 0, 70, 90, 110, 127, 37, 37, false },
      { QString("Acoustic Snare"),     16, 32, 9, 0, 70, 90, 110, 127, 38, 38, false },
      { QString("Hand Clap"),          16, 32, 9, 0, 70, 90, 110, 127, 39, 39, false },
      { QString("Electric Snare"),     16, 32, 9, 0, 70, 90, 110, 127, 40, 40, false },
      { QString("Low Floor Tom"),      16, 32, 9, 0, 70, 90, 110, 127, 41, 41, false },
      { QString("Closed Hi-Hat"),      16, 32, 9, 0, 70, 90, 110, 127, 42, 42, false },
      { QString("High Floor Tom"),     16, 32, 9, 0, 70, 90, 110, 127, 43, 43, false },
      { QString("Pedal Hi-Hat"),       16, 32, 9, 0, 70, 90, 110, 127, 44, 44, false },
      { QString("Low Tom"),            16, 32, 9, 0, 70, 90, 110, 127, 45, 45, false },
      { QString("Open Hi-Hat"),        16, 32, 9, 0, 70, 90, 110, 127, 46, 46, false },
      { QString("Low-Mid Tom"),        16, 32, 9, 0, 70, 90, 110, 127, 47, 47, false },
      { QString("Hi-Mid Tom"),         16, 32, 9, 0, 70, 90, 110, 127, 48, 48, false },
      { QString("Crash Cymbal 1"),     16, 32, 9, 0, 70, 90, 110, 127, 49, 49, false },
      { QString("High Tom"),           16, 32, 9, 0, 70, 90, 110, 127, 50, 50, false },

      { QString("Ride Cymbal 1"),      16, 32, 9, 0, 70, 90, 110, 127, 51, 51, false },
      { QString("Chinese Cymbal"),     16, 32, 9, 0, 70, 90, 110, 127, 52, 52, false },
      { QString("Ride Bell"),          16, 32, 9, 0, 70, 90, 110, 127, 53, 53, false },
      { QString("Tambourine"),         16, 32, 9, 0, 70, 90, 110, 127, 54, 54, false },
      { QString("Splash Cymbal"),      16, 32, 9, 0, 70, 90, 110, 127, 55, 55, false },
      { QString("Cowbell"),            16, 32, 9, 0, 70, 90, 110, 127, 56, 56, false },
      { QString("Crash Cymbal 2"),     16, 32, 9, 0, 70, 90, 110, 127, 57, 57, false },
      { QString("Vibraslap"),          16, 32, 9, 0, 70, 90, 110, 127, 58, 58, false },
      { QString("Ride Cymbal 2"),      16, 32, 9, 0, 70, 90, 110, 127, 59, 59, false },
      { QString("Hi Bongo"),           16, 32, 9, 0, 70, 90, 110, 127, 60, 60, false },
      { QString("Low Bongo"),          16, 32, 9, 0, 70, 90, 110, 127, 61, 61, false },
      { QString("Mute Hi Conga"),      16, 32, 9, 0, 70, 90, 110, 127, 62, 62, false },
      { QString("Open Hi Conga"),      16, 32, 9, 0, 70, 90, 110, 127, 63, 63, false },
      { QString("Low Conga"),          16, 32, 9, 0, 70, 90, 110, 127, 64, 64, false },
      { QString("High Timbale"),       16, 32, 9, 0, 70, 90, 110, 127, 65, 65, false },
      { QString("Low Timbale"),        16, 32, 9, 0, 70, 90, 110, 127, 66, 66, false },

      { QString("High Agogo"),         16, 32, 9, 0, 70, 90, 110, 127, 67, 67, false },
      { QString("Low Agogo"),          16, 32, 9, 0, 70, 90, 110, 127, 68, 68, false },
      { QString("Cabasa"),             16, 32, 9, 0, 70, 90, 110, 127, 69, 69, false },
      { QString("Maracas"),            16, 32, 9, 0, 70, 90, 110, 127, 70, 70, false },
      { QString("Short Whistle"),      16, 32, 9, 0, 70, 90, 110, 127, 71, 71, false },
      { QString("Long Whistle"),       16, 32, 9, 0, 70, 90, 110, 127, 72, 72, false },
      { QString("Short Guiro"),        16, 32, 9, 0, 70, 90, 110, 127, 73, 73, false },
      { QString("Long Guiro"),         16, 32, 9, 0, 70, 90, 110, 127, 74, 74, false },
      { QString("Claves"),             16, 32, 9, 0, 70, 90, 110, 127, 75, 75, false },
      { QString("Hi Wood Block"),      16, 32, 9, 0, 70, 90, 110, 127, 76, 76, false },
      { QString("Low Wood Block"),     16, 32, 9, 0, 70, 90, 110, 127, 77, 77, false },
      { QString("Mute Cuica"),         16, 32, 9, 0, 70, 90, 110, 127, 78, 78, false },
      { QString("Open Cuica"),         16, 32, 9, 0, 70, 90, 110, 127, 79, 79, false },
      { QString("Mute Triangle"),      16, 32, 9, 0, 70, 90, 110, 127, 80, 80, false },
      { QString("Open Triangle"),      16, 32, 9, 0, 70, 90, 110, 127, 81, 81, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 82, 82, false },

      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 83, 83, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 84, 84, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 85, 85, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 86, 86, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 87, 87, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 88, 88, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 89, 89, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 90, 90, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 91, 91, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 92, 92, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 93, 93, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 94, 94, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 95, 95, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 96, 96, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 97, 97, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 98, 98, false },

      /*
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 83, 83, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 84, 84, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 85, 85, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 86, 86, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 87, 87, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 88, 88, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 89, 89, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 90, 90, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 91, 91, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 92, 92, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 93, 93, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 94, 94, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 95, 95, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 96, 96, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 97, 97, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 98, 98, false },

      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 83, 83, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 84, 84, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 85, 85, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 86, 86, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 87, 87, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 88, 88, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 89, 89, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 90, 90, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 91, 91, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 92, 92, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 93, 93, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 94, 94, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 95, 95, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 96, 96, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 97, 97, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 98, 98, false },

      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 83, 83, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 84, 84, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 85, 85, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 86, 86, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 87, 87, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 88, 88, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 89, 89, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 90, 90, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 91, 91, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 92, 92, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 93, 93, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 94, 94, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 95, 95, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 96, 96, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 97, 97, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 98, 98, false },

      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 83, 83, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 84, 84, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 85, 85, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 86, 86, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 87, 87, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 88, 88, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 89, 89, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 90, 90, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 91, 91, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 92, 92, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 93, 93, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 94, 94, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 95, 95, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 96, 96, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 97, 97, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 98, 98, false }
      };
      */

      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 99, 99, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 100, 100, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 101, 101, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 102, 102, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 103, 103, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 104, 104, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 105, 105, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 106, 106, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 107, 107, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 108, 108, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 109, 109, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 110, 110, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 111, 111, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 112, 112, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 113, 113, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 114, 114, false },

      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 115, 115, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 116, 116, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 117, 117, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 118, 118, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 119, 119, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 120, 120, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 121, 121, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 122, 122, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 123, 123, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 124, 124, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 125, 125, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 126, 126, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 127, 127, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 0, 0, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 1, 1, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 2, 2, false },

      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 3, 3, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 4, 4, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 5, 5, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 6, 6, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 7, 7, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 8, 8, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 9, 9, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 10, 10, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 11, 11, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 12, 12, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 13, 13, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 14, 14, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 15, 15, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 16, 16, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 17, 17, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 18, 18, false },

      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 19, 19, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 20, 20, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 21, 21, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 22, 22, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 23, 23, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 24, 24, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 25, 25, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 26, 26, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 27, 27, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 28, 28, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 29, 29, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 30, 30, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 31, 31, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 32, 32, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 33, 33, false },
      { QString(""),                   16, 32, 9, 0, 70, 90, 110, 127, 34, 34, false }
      };
      

//---------------------------------------------------------
//   initDrumMap
//    populate Inmap and Outmap
//---------------------------------------------------------

void initDrumMap()
      {
      for (int i = 0; i < DRUM_MAPSIZE; ++i) {
            DrumMap d = drumMap[i];
            //Make sure we're not overwriting any values loaded
            //On init, all these values are zero. If so, just set the drummap entry to the initial drummap entry.
            if (!(d.len || d.channel || d.port || d.lv1 || d.lv2 || d.lv3 || d.lv4 || d.enote || d.anote || d.mute))
                  drumMap[i] = idrumMap[i];
         }
      //Finally, setup the inMap, outMap-values
      memset(drumInmap, 0, sizeof(drumInmap));
      memset(drumOutmap, 0, sizeof(drumOutmap));
      for (int i = 0; i < DRUM_MAPSIZE; ++i) {
            drumInmap[(unsigned int)(drumMap[i].enote)] = i;
            drumOutmap[(unsigned int)(drumMap[i].anote)] = i;
            }
      }

//---------------------------------------------------------
//   resetGMDrumMap
//---------------------------------------------------------

void resetGMDrumMap()
      {
      audio->msgIdle(true);
      // Delete all port controller events.
      //audio->msgChangeAllPortDrumCtrlEvents(false);
      song->changeAllPortDrumCtrlEvents(false);
      
      for(int i = 0; i < DRUM_MAPSIZE; ++i) 
        drumMap[i] = idrumMap[i];
      memset(drumInmap, 0, sizeof(drumInmap));
      memset(drumOutmap, 0, sizeof(drumOutmap));
      for (int i = 0; i < DRUM_MAPSIZE; ++i) {
            drumInmap[(unsigned int)(drumMap[i].enote)] = i;
            drumOutmap[(unsigned int)(drumMap[i].anote)] = i;
            }
      // Add all port controller events.
      //audio->msgChangeAllPortDrumCtrlEvents(true);
      song->changeAllPortDrumCtrlEvents(true);
      audio->msgIdle(false);
      }

//---------------------------------------------------------
//   operator ==
//---------------------------------------------------------

bool const DrumMap::operator==(const DrumMap& map) const
      {
      return
         (name == map.name)
         && quant == map.quant
         && len == map.len
         && channel == map.channel
         && port == map.port
         && lv1 == map.lv1
         && lv2 == map.lv2
         && lv3 == map.lv3
         && lv4 == map.lv4
         && enote == map.enote
         && anote == map.anote
         && mute == map.mute;
      }

//---------------------------------------------------------
//   writeDrumMap
//---------------------------------------------------------

void writeDrumMap(int level, Xml& xml, bool external)
      {
      xml.tag(level++, "drummap");
      for (int i = 0; i < DRUM_MAPSIZE; ++i) {
            DrumMap* dm = &drumMap[i];
            const DrumMap* idm = &idrumMap[i];

            if (external) {
                  xml.tag(level++, "entry");
                  xml.strTag(level, "name", dm->name);
                  xml.intTag(level, "quant", dm->quant);
                  xml.intTag(level, "len", dm->len);
                  xml.intTag(level, "channel", dm->channel);
                  xml.intTag(level, "port", dm->port);
                  xml.intTag(level, "lv1", dm->lv1);
                  xml.intTag(level, "lv2", dm->lv2);
                  xml.intTag(level, "lv3", dm->lv3);
                  xml.intTag(level, "lv4", dm->lv4);
                  xml.intTag(level, "enote", dm->enote);
                  xml.intTag(level, "anote", dm->anote);
                  }
            else {
                  // write only, if entry is different from initial entry
                  if (!external && *dm == *idm)
                        continue;
                  xml.tag(level++, "entry idx=\"%d\"", i);
                  if (dm->name != idm->name)
                        xml.strTag(level, "name", dm->name);
                  if (dm->quant != idm->quant)
                        xml.intTag(level, "quant", dm->quant);
                  if (dm->len != idm->len)
                        xml.intTag(level, "len", dm->len);
                  if (dm->channel != idm->channel)
                        xml.intTag(level, "channel", dm->channel);
                  if (dm->port != idm->port)
                        xml.intTag(level, "port", dm->port);
                  if (dm->lv1 != idm->lv1)
                        xml.intTag(level, "lv1", dm->lv1);
                  if (dm->lv2 != idm->lv2)
                        xml.intTag(level, "lv2", dm->lv2);
                  if (dm->lv3 != idm->lv3)
                        xml.intTag(level, "lv3", dm->lv3);
                  if (dm->lv4 != idm->lv4)
                        xml.intTag(level, "lv4", dm->lv4);
                  if (dm->enote != idm->enote)
                        xml.intTag(level, "enote", dm->enote);
                  if (dm->anote != idm->anote)
                        xml.intTag(level, "anote", dm->anote);
                  if (dm->mute != idm->mute)
                        xml.intTag(level, "mute", dm->mute);
                  }
            xml.tag(level--, "/entry");
            }
      xml.tag(level--, "/drummap");
      }

//---------------------------------------------------------
//   readDrummapEntry
//---------------------------------------------------------

static void readDrummapEntry(Xml& xml, DrumMap* dm)
      {
      
      for (;;) {
            Xml::Token token = xml.parse();
            const QString& tag = xml.s1();
            switch (token) {
                  case Xml::Error:
                  case Xml::End:
                        return;
                  case Xml::TagStart:
                        if (tag == "name")
                              dm->name = xml.parse(QString("name"));
                        else if (tag == "quant")
                              dm->quant = xml.parseInt();
                        else if (tag == "len")
                              dm->len = xml.parseInt();
                        else if (tag == "channel")
                              dm->channel = xml.parseInt();
                        else if (tag == "port")
                              dm->port = xml.parseInt();
                        else if (tag == "lv1")
                              dm->lv1 = xml.parseInt();
                        else if (tag == "lv2")
                              dm->lv2 = xml.parseInt();
                        else if (tag == "lv3")
                              dm->lv3 = xml.parseInt();
                        else if (tag == "lv4")
                              dm->lv4 = xml.parseInt();
                        else if (tag == "enote")
                              dm->enote = xml.parseInt();
                        else if (tag == "anote")
                              dm->anote = xml.parseInt();
                        else if (tag == "mute")
                              dm->mute = xml.parseInt();
                        else if (tag == "selected")
                              //; // dm->selected = xml.parseInt();
                              xml.skip(tag);
                        else
                              xml.unknown("DrumMapEntry");
                        break;
                  case Xml::Attribut:
                        if (tag == "idx") {
                              int idx = xml.s2().toInt() & 0x7f;
                              dm = &drumMap[idx];
                              
                              }
                        break;
                  case Xml::TagEnd:
                        if (tag == "entry")
                        {
                              return;
                        }      
                  default:
                        break;
                  }
            }
      }

//---------------------------------------------------------
//   readDrummap
//---------------------------------------------------------

void readDrumMap(Xml& xml, bool external)
      {
      audio->msgIdle(true);
      // Delete all port controller events.
      //audio->msgChangeAllPortDrumCtrlEvents(false);
      song->changeAllPortDrumCtrlEvents(false);
      
      if (external) {
            for (int i = 0; i < DRUM_MAPSIZE; ++i)
                  drumMap[i] = blankdm;
            }
      else {
            for (int i = 0; i < DRUM_MAPSIZE; ++i)
                  drumMap[i] = idrumMap[i];
            }
      int i = 0;
      for (;;) {
            Xml::Token token = xml.parse();
            const QString& tag = xml.s1();
            switch (token) {
                  case Xml::Error:
                  case Xml::End:
                        audio->msgIdle(false);
                        return;
                  case Xml::TagStart:
                        if (tag == "entry") {
                              if(i >= DRUM_MAPSIZE)
                              {
                                audio->msgIdle(false);
                                return;
                              }
                              readDrummapEntry(xml, external ? &drumMap[i] : 0);
                              ++i;
                              }
                        else if (tag == "comment")
                              xml.parse();
                        else
                              xml.unknown("DrumMap");
                        break;
                  case Xml::Attribut:
                        break;
                  case Xml::TagEnd:
                        if (tag == "drummap") {
                              memset(drumInmap, 0, sizeof(drumInmap));
                              memset(drumOutmap, 0, sizeof(drumOutmap));
                              for (int i = 0; i < DRUM_MAPSIZE; ++i) {
                                    drumInmap[(unsigned int)(drumMap[i].enote)] = i;
                                    drumOutmap[(unsigned int)(drumMap[i].anote)] = i;
                                    }
                              // Add all port controller events.
                              //audio->msgChangeAllPortDrumCtrlEvents(true);
                              song->changeAllPortDrumCtrlEvents(true);
                              
                              audio->msgIdle(false);
                              return;
                              }
                  default:
                        break;
                  }
            }
            // Add all port controller events.
            //audio->msgChangeAllPortDrumCtrlEvents(true);
            song->changeAllPortDrumCtrlEvents(true);
            audio->msgIdle(false);
      }

